/*********************************************************/
/*  This source code is copyrighted by the author        */
/*  Please read COPYING  for the GNU licence policy.     */
/*********************************************************/
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "tetX.h"
#include "egafont.h"

extern Display  *display;
extern Visual   *visual;
extern GC	graphContext;
extern Pixmap	screenContext;

extern int	ScreenDepth;
extern Bool	DoInvalidate;
extern unsigned long PixValues[];


static char *FontData;
static int  FontHeight;


int egaShowText(char *text, int x, int y, int fgcol, int bgcol)
{  
   XImage	*xim = NULL;    
   unsigned char *data = NULL;
   unsigned char mask;
   unsigned int  txtwidth;
   int		 x1, x2, y1;
   unsigned char fbyte;
   unsigned char *fontptr, textchar;
   unsigned long fgrPix, bgrPix;

   fgrPix = PixValues[fgcol];
   bgrPix = PixValues[bgcol];  

   txtwidth = strlen(text) * 8;    

   xim  = XCreateImage(display, visual, ScreenDepth,
                       ZPixmap, 0, NULL, 
                       txtwidth, FontHeight, 8, 0);
   if (!xim) goto Finito;
   
   data = malloc(xim->bytes_per_line * FontHeight);
   if (!data) goto Finito;
   xim->data = data;

   x1 = 0;
   while ((textchar = *text++) != '\0')
   {  
      fontptr = FontData + textchar*FontHeight;

      for (y1=0; y1<FontHeight; y1++)      
      {  fbyte = *fontptr++;
	 mask = 0x80;              
         x2 = x1;
	 while (mask)
	 {  XPutPixel (xim, x2, y1, (fbyte & mask) ? fgrPix: bgrPix);
	    x2++;
            mask = mask >> 1;
	 }
      }	   

      x1+=8;
   }      

   XPutImage(display, screenContext, graphContext, xim,
          0, 0, x, y, txtwidth, FontHeight);

   if (DoInvalidate)
   { RECT rect;
       rect.left = x; rect.right = x+txtwidth;
       rect.top = y;  rect.bottom = y+FontHeight;
      ContextRectToScreen(&rect);
   }     	  


Finito:   
   if (xim) XDestroyImage(xim);   
   return 0;

}

int egaShowScaledText(char *text, int x, int y,
                      int fgcol, int bgcol, int hscale, int vscale)
{

   XImage	 *xim = NULL;    
   unsigned char *data = NULL;
   unsigned char mask;
   signed   int  x1, y1, x2, y2, x3, y3, cx, cy;
   unsigned int  textwid, scalewid, scalehei;
   unsigned char fbyte;
   unsigned char *fontptr;
   unsigned char textchar;
   unsigned long fgrPix, bgrPix, curPix;


   fgrPix = PixValues[fgcol];
   bgrPix = PixValues[bgcol];  

   textwid  = strlen(text) * 8;    
   scalewid = textwid  * hscale;
   scalehei = FontHeight * vscale;

   xim  = XCreateImage(display, visual, ScreenDepth,
                       ZPixmap, 0, NULL, 
                       scalewid, scalehei, 8, 0);
   if (!xim) goto Finito;
   
   data = malloc(xim->bytes_per_line * scalehei);
   if (!data) goto Finito;
   xim->data = data;

   x1 = 0;
   while ((textchar = *text++) != '\0')
   {  
      fontptr = FontData + textchar*FontHeight;
      
      y1 = 0;
      y2 = FontHeight;
      while (--y2>=0)      
      {  fbyte = *fontptr++;
	 mask = 0x80;              
         x2 = x1;
	 while (mask)
	 {  y3 = y1;
	    curPix = (fbyte & mask) ? fgrPix: bgrPix;
	    cy = vscale;
	    while(--cy >= 0)
	    {  cx = hscale;
	       x3 = x2;
	       while (--cx>=0)
	       {  XPutPixel (xim, x3, y3, curPix);
	          x3++;
	       }
	       y3++;
	    }   	  
	    x2 = x3;
            mask = mask >> 1;
	 }
	 
	 y1 = y3;
      }	   

      x1 = x2;
   }      

   XPutImage(display, screenContext, graphContext, xim,
          0, 0, x, y, scalewid, scalehei);

   if (DoInvalidate)
   {  RECT rect;
      rect.left = x; rect.right = x+scalewid;
      rect.top = y;  rect.bottom = y+scalehei;
      ContextRectToScreen(&rect);
   }     	  

Finito:   
   if (xim)  XDestroyImage(xim);   
   return 0;


}

int egaSetFont(char *cFont)
{   int *iFont;
    if (!cFont) return 4;

    iFont = (int *)cFont;
    FontHeight = *iFont;
    
    FontData = cFont + sizeof(int);
    return 0;
}
  
void egaFreeFont(char *cFont)
{
   if (!cFont) return;

   if (cFont + sizeof(int) == FontData)
   {  FontData = NULL; FontHeight = 0; }

   free(cFont);
}        


int egaGetFontHeight(void)
{   return FontHeight; }

int egaGetTextWidth(char *text)
{  return strlen(text) * 8; }

FONT_TYPE egaFontType(void)
{
   return FONT_TYPE_X11_CONSOLE;
}
/*
char *egaGetFontPtr(int chr)
{  return FontData + (chr & 0xff) *FontHeight; }
*/
char *egaLoadFont(char *fontFileName, char *fontDirName, char *mes)
{
   int fd;
   int rc = 4; 
   size_t s;
   off_t  hdrlen;    // File header length

   char  *cFont = NULL;
   int   *iFont;
   char *FontPath, *FontPath1, *FontPath2;
   int   fontheight;

   int   unzipped;
   char  *tmp_file_pref = "/tmp/itetris";
   char tmp_file_name[40];
   unsigned char strt[4];

   FontPath = malloc(257);
   strcpy(FontPath, fontDirName);
   fontheight = strlen(FontPath);
   if (FontPath[fontheight-1] != '/') strcat(FontPath, "/");
   strcat(FontPath, fontFileName);

   FontPath1 = GetFullFileName(FontPath);    

   fontheight = strlen(FontPath1);
   unzipped = strcasecmp(FontPath1+fontheight-3, ".gz");

   fd = open(FontPath1, O_RDONLY);

   if (fd < 0 && unzipped)
   {
      FontPath2 = malloc(strlen(FontPath1) + 4);
      sprintf (FontPath2, "%s.gz", FontPath1);     
      free(FontPath1); FontPath1 = FontPath2;	
      unzipped = 0;
      fd = open(FontPath1, O_RDONLY);
   }

   if (fd >= 0 && !unzipped)
   {
      close(fd);
      sprintf( tmp_file_name, "%s.%lu",
               tmp_file_pref, (unsigned long) getpid());
      FontPath2 = malloc(strlen(FontPath1) + strlen(tmp_file_name) + 17);
      sprintf(FontPath2, "gunzip -c %s > %s", FontPath1, tmp_file_name);
      system(FontPath2);
      free(FontPath2);

      fd = open(tmp_file_name, O_RDONLY);
   }       
   free(FontPath1);

   if (fd < 0) sprintf(mes, "Could not open font %s\n", FontPath);
   free(FontPath);
   
   if (fd < 0) return NULL;

   /* Try to find the size of characters */
   read(fd, strt, 4);

   if (strt[0] == 0x36 && strt[1] == 4 && strt[2] <= 1)
   {  /* psf file */
        fontheight = strt[3];
        s = fontheight * 256;
   }
   else
   {   /* Size of the font file */
       s = (size_t) lseek(fd, 0, SEEK_END);

       fontheight = s>>8;   /* Assume 256 characters, FontHeight bytes each */
       hdrlen  = s & 255;
       if (hdrlen != 0 && hdrlen != 4) goto Ret;
       
       s -= hdrlen;
       lseek(fd, hdrlen, SEEK_SET);  /* Bypassing header, if needed */
   }

   cFont = (char *) malloc(s + sizeof(int));
   if (!cFont) goto Ret;

   iFont = (int *)cFont;
   *iFont = fontheight;

   if (read(fd, cFont+sizeof(int), s) == s) rc = 0; 
 
Ret:  
   
   close(fd);
   if (!unzipped) remove(tmp_file_name);

   if (rc)
   {  if (cFont) free(cFont);
      return NULL;
   }

   return cFont;
}


int egaShowCharSpecial(char b, int x, int y, int fgcol, int bgcol,
                       int hscale, int vscale, int flags)
{
    XImage	 *xim = NULL;    
    unsigned char *data = NULL;
    unsigned char mask;
   

    signed   int  y1, x2, y2, x3, y3, cx, cy;
    signed   int  xstart, xincr, ystart, yincr;

    unsigned int  scalewid, scalehei;
    unsigned char fbyte;
    unsigned char *fontptr;
    unsigned long fgrPix, bgrPix, curPix;


    fgrPix = PixValues[fgcol];
    bgrPix = PixValues[bgcol];  

    scalewid = hscale * 8;
    scalehei = FontHeight * vscale;

    xim  = XCreateImage(display, visual, ScreenDepth,
                       ZPixmap, 0, NULL, 
                       scalewid, scalehei, 8, 0);
    if (!xim) goto Finito;
   
    data = malloc(xim->bytes_per_line * scalehei);
    if (!data) goto Finito;
    xim->data = data;
    
    if (flags & EGAFONT_HFLIP)
    {   xstart =  FontHeight - 1;
        xincr = -1;
    }
    else
    {  xstart = 0;
       xincr = 1;
    }   

    if (flags & EGAFONT_VFLIP)
    {   ystart =  FontHeight - 1;
        yincr = -1;
    }
    else
    {  ystart = 0;
       yincr = 1;
    }   


    b &= 0xff;
    fontptr = FontData + b*FontHeight;
    y1 = ystart*vscale;
    yincr *= vscale;
    xstart *= hscale;
    xincr *= hscale;

    y2 = FontHeight;
    while (--y2>=0)      
    {    fbyte = *fontptr++;
	 mask = 0x80;              
         x2 = xstart;
	 while (mask)
	 {  y3 = y1;
	    curPix = (fbyte & mask) ? fgrPix: bgrPix;
	    cy = vscale;
	    while(--cy >= 0)
	    {  cx = hscale;
	       x3 = x2;
	       while (--cx>=0)
	       {  XPutPixel (xim, x3, y3, curPix);
	          x3++;
	       }
	       y3++;
	    }   	  
	    x2 += xincr;
            mask = mask >> 1;
	 }
	 
	 y1 += yincr;
     }	   
    


   XPutImage(display, screenContext, graphContext, xim,
          0, 0, x, y, scalewid, scalehei);

   if (DoInvalidate)
   {  RECT rect;
      rect.left = x; rect.right = x+scalewid;
      rect.top = y;  rect.bottom = y+scalehei;
      ContextRectToScreen(&rect);
   }     	  

Finito:   
   if (xim)  XDestroyImage(xim);   
   return 0;
     
} 

XImage *DrawCharOnXImage(char textchar, unsigned long fgrPix, unsigned long bgrPix)
{  

   XImage *xim;
   unsigned char *data = NULL;
   unsigned char mask;
   int		 x2, y1;
   unsigned char fbyte;
   unsigned char *fontptr;

   if (!FontData) return NULL;


   xim  = XCreateImage(display, visual, ScreenDepth,
                       ZPixmap, 0, NULL, 
                       8, FontHeight, 8, 0);
   if (!xim) return NULL;
      
   data = malloc(xim->bytes_per_line * FontHeight);
   if (!data) 
   { XDestroyImage(xim); return NULL ; }
   
   xim->data = data;

   fontptr = FontData + textchar*FontHeight;

   for (y1=0; y1<FontHeight; y1++)       
   {  fbyte = *fontptr++;
      mask = 0x80;              
      x2 = 0;
      while (mask)
      {  XPutPixel (xim, x2, y1, (fbyte & mask) ? fgrPix: bgrPix);
          x2++;
          mask = mask >> 1;
      }
   }

   return xim;
}
