/*********************************************************/
/*  This source code is copyrighted by the author        */
/*  Please read COPYING  for the GNU licence policy.     */
/*********************************************************/
#include <vga.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "tet.h"
#include "egafont.h"
static char *Font;
static int  FontHeight;

int egaShowText(char *text, int x, int y, int fgcol, int bgcol)
{

   unsigned char buff[160];		// Enough for 20 char-s
   unsigned char *buffer;
   unsigned char mask;
   int  buflen,i;
   unsigned char fbyte;
   unsigned char *to;
   unsigned char *textptr, textchar;
 
   buflen = strlen(text) << 3;
   // Attempting to save time for short texts
   
   if (buflen > 160)
     {  buffer = (char *) malloc(buflen);
        if (!buffer) return 4;
     } 
   else  buffer = buff;

   for (i=0; i<FontHeight; i++)
   { 
      textptr = text;
      to = buffer;
      while ((textchar = *textptr++) != '\0')
      {  fbyte = *(Font + textchar*FontHeight + i);
         mask = 0x80;
         while(mask)
         {  *to++ = (fbyte & mask) ? fgcol: bgcol;
            mask = mask >> 1;
         }
      }
    
      setrow(x, y+i, buflen, buffer);
   }
   
   if (buflen > 160) free (buffer);

   return 0;

}

int egaShowScaledText(char *text, int x, int y,
                      int fgcol, int bgcol, int hscale, int vscale)
{

   unsigned char *buffer;
   unsigned char mask;
   int  buflen,i, j;
   unsigned char fbyte, c;
   unsigned char *to;
   unsigned char *textptr,textchar;

   buflen = (strlen(text) << 3) * hscale;
   
   //  This is called rarely, so you don't have to worry about time 
   buffer = (char *) malloc(buflen);

   for (i=0; i<FontHeight; i++)
   {
      textptr = text;
      to = buffer;
      while ((textchar = *textptr++) != '\0')
      {  fbyte = *(Font + textchar*FontHeight + i);
         mask = 0x80;
 
         while(mask)
         {  c =  (fbyte & mask) ? fgcol: bgcol;
            j = hscale;
            while(--j >=0) *to++ = c;
            mask = mask >> 1;
         }
      }
      
      j = vscale;
      while (--j >= 0)
         setrow(x, y++, buflen, buffer);
   }
   free (buffer);


   return 0;

}

int egaSetFont(char *cFont)
{   int *iFont;
    if (!cFont) return 4;

    iFont = (int *)cFont;
    FontHeight = *iFont;
    
    Font = cFont + sizeof(int);
    return 0;
}
  
void egaFreeFont(char *cFont)
{
   if (!cFont) return;

   if (cFont + sizeof(int) == Font)
   {  Font = NULL; FontHeight = 0; }

   free(cFont);
}        


int egaGetFontHeight(void)
{   return FontHeight; }

int egaGetTextWidth(char *text)
{  return strlen(text) * 8; }

char *egaGetFontPtr(int chr)
{  return Font + (chr & 0xff) *FontHeight; }


char *egaLoadFont(char *fontFileName, char *fontDirName, char *mes)
{
   int fd; 
 
   int rc = 4; 
   size_t s;
   off_t  hdrlen;    // File header length

   char  *cFont = NULL;
   int   *iFont;
   char *FontPath, *FontPath1, *FontPath2;
   unsigned int fontheight;

   int   unzipped;
   static const char *tmp_file_pref = "itet";
   char  *tmp_file_name = NULL;
   unsigned char  strt[4];

   FontPath = malloc(257);
   strcpy(FontPath, fontDirName);
   fontheight = strlen(FontPath);
   if (FontPath[fontheight-1] != '/') strcat(FontPath, "/");
   strcat(FontPath, fontFileName);

   FontPath1 = GetFullFileName(FontPath);    


   fontheight = strlen(FontPath1);
   unzipped = strcasecmp(FontPath1+fontheight-3, ".gz");

   fd = open(FontPath1, O_RDONLY);

   if (fd < 0 && unzipped)
   {
      FontPath2 = malloc(strlen(FontPath1) + 4);
      sprintf (FontPath2, "%s.gz", FontPath1);     
      free(FontPath1); FontPath1 = FontPath2;	
      unzipped = 0;
      fd = open(FontPath1, O_RDONLY);
   }

   if (fd >= 0 && !unzipped)
   {
      close(fd); fd = -1;
      tmp_file_name = tempnam(NULL, tmp_file_pref);
      if (tmp_file_name)
      {		
         FontPath2 = malloc(strlen(FontPath1) + strlen(tmp_file_name) + 17);
         sprintf(FontPath2, "gunzip -c %s > %s", FontPath1, tmp_file_name);
         system(FontPath2);
         free(FontPath2);

         fd = open(tmp_file_name, O_RDONLY);
      }
   }       

   free(FontPath1);

   if (fd < 0) sprintf(mes, "Could not open font %s\n", FontPath);
   free(FontPath);
   
   if (fd < 0) return NULL;

   /* Try to find the size of characters */
   read(fd, strt, 4);

   if (strt[0] == 0x36 && strt[1] == 4 && strt[2] <= 1)
   {  /* psf file */
        fontheight = strt[3];
        s = fontheight * 256;
   }
   else
   {   /* Size of the font file */
       s = (size_t) lseek(fd, 0, SEEK_END);

       fontheight = s>>8;       // Assume 256 characters, FontHeight bytes each
       hdrlen  = s & 255;
       if (hdrlen != 0 && hdrlen != 4) goto Ret;
       s -= hdrlen;

       lseek(fd, hdrlen, SEEK_SET);  // Bypassing header
   }

   cFont = (char *) malloc(s + sizeof(int));
   if (!cFont) goto Ret;

   iFont = (int *)cFont;
   *iFont = fontheight;

   if (read(fd, cFont+sizeof(int), s) == s) rc = 0; 
 
Ret:  
 
   close(fd);
   if (tmp_file_name)
   {   remove(tmp_file_name);
       free(tmp_file_name);
   }

   if (rc)
   {  if (cFont) free(cFont);
      return NULL;
   }

   return cFont;
}

int egaShowCharSpecial(char b, int x, int y, int fgcol, int bgcol,
                       int hscale, int vscale, int flags)
{   char *buffer, *bufptr, c;
    unsigned char *fontptr, fbyte, mask, smask;
    int hflip;
    int i, y1, yincr, w;
    
    w = hscale << 3; 
    buffer = (char *) malloc(w);
    if (!buffer) return 4;
 
    hflip = flags & EGAFONT_HFLIP;
    if (flags & EGAFONT_VFLIP)
    { y += vscale * FontHeight - 1;
      yincr = -1;
    }
    else
      yincr = 1;
    
    smask = hflip? 1 : 0x80;
    
    b &= 0xff;
    fontptr = Font + b*FontHeight;
 
    y1 = FontHeight;
    while(--y1 >= 0)
    {  
        fbyte = *fontptr++;
        bufptr = buffer;
        mask = smask;
	while (mask)  
        {  c = fbyte & mask ? fgcol : bgcol;
           for ( i=0; i<hscale; i++) *bufptr++ = c;
           if (hflip) mask <<=1; else mask >>= 1; 
        }   

        for (i=0; i<vscale; i++) 
        {   setrow(x, y, w, buffer);
            y += yincr;
        }
     }

     free(buffer);
     return 0; 
} 
