/*********************************************************/
/*  This source code is copyrighted by the author        */
/*  Please read COPYING  for the GNU licence policy.     */
/*********************************************************/
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "tetX.h"
#include "egafont.h"

extern  Display *display;
extern  Pixmap  screenContext;
extern  Window  window;
extern  int	ScreenDepth;

extern  GC      graphContext;
extern  Bool    DoInvalidate;

static XFontStruct *FontData; 
static int  FontHeight;

extern COLOUR_REF  PixValues[];

int egaShowText(char *text, int x, int y, int fgcol, int bgcol)
{  XGCValues xgcv;
   int  len = strlen(text);

   if (!FontData) return 1;
   xgcv.foreground = PixValues[fgcol];
   xgcv.background = PixValues[bgcol];
  
   XChangeGC(display, graphContext, GCForeground | GCBackground ,
                                     &xgcv);

   XDrawImageString (display, screenContext, graphContext,
                    x, y+FontData->ascent, text, len);

   if (DoInvalidate)
   { RECT rect;

     rect.left = x; 
     rect.top = y; //FontData->descent;
     rect.bottom = y + FontHeight;
     rect.right = rect.left + XTextWidth(FontData, text, len);
     ContextRectToScreen(&rect);
   }     

   return 0;

}

int egaShowScaledText(char *text, int x, int y,
                      int fgcol, int bgcol, int hscale, int vscale)
{
    int wid, hei, thei, twid;
    int len;
    XGCValues xgcv;
    XImage   *xim;
    int      x1, y1, x2, y2, x3, y3, i, j;
    COLOUR_REF pixel;
     
    if (!FontData) return 1;

    len = strlen(text);
    wid = XTextWidth(FontData, text, len);
    hei = FontHeight;
    twid = wid * hscale;
    thei = hei * vscale;
    
    xgcv.foreground = PixValues[fgcol];
    xgcv.background = PixValues[bgcol];
  
    XChangeGC(display, graphContext, GCForeground | GCBackground ,
                                     &xgcv);

    XDrawImageString (display, screenContext, graphContext,
                    x, y+FontData->ascent, text, len);


    xim = XGetImage(display, screenContext, x, y,
                          twid, thei, AllPlanes, ZPixmap);

    if (!xim) return 2;
    
        
    /* Now scale! */
    for (y3=hei-1; y3>=0; y3--)
    {  y1 = y3*vscale;

       // From a single line
       for (x3=wid-1; x3>=0; x3--)
       {  pixel = XGetPixel(xim, x3, y3);

	  
	  // Spread to whole square          
	  x1 = x3 * hscale;
	  y2 = y1;
	  for (j = 0; j < vscale; j++)
	  {   x2 = x1;
	      for (i=0; i < hscale; i++)
	         XPutPixel(xim, x2++, y2, pixel);
	      y2++;	 
	  }      
	  
       }
    }   
             	       	       

    XPutImage(display, screenContext, graphContext, xim, 0, 0,
                       x, y, twid, thei);

    XDestroyImage(xim);
            
   if (DoInvalidate)
   { RECT rect;

     rect.left = x; 
     rect.top = y; //FontData->descent;
     rect.bottom = y + thei;
     rect.right = rect.left + twid;
     ContextRectToScreen(&rect);
   }     

   return 0;
    

}

int egaSetFont(char *cFontData)
{   
    FontData = (XFontStruct *) cFontData;

    if (FontData)
    {    XSetFont(display, graphContext, FontData->fid);
	 FontHeight = FontData->ascent + FontData->descent;
    }	 
    else
        FontHeight = 0;
    
   return 0;
}

void egaFreeFont(char *cFont)
{
   if (cFont)
      XFreeFont(display, (XFontStruct *) cFont);
}


int egaGetFontHeight(void)
{
     return  FontHeight;
}

int egaGetTextWidth(char *text)
{   if (!FontData) return 0;

    return XTextWidth(FontData, text, strlen(text));
}

FONT_TYPE egaFontType(void)
{
   return FONT_TYPE_X11_NATURAL;
}

/*
char *egaGetFontPtr(int chr)
{
    return (char *) FontData;   // ???? not for a partic char
}
*/

Bool GetFontInfoAndCheck(XFontStruct *xfs, char *target)
{
    char *Foundry = NULL;
    char *Charset = NULL;
    char *Weight = NULL;
    unsigned long   wid, hei;
    Atom atm;
    unsigned long value; 
    Bool	status;
/*    
    int  i;
    
    for (i=0; i<xfs->n_properties; i++)
    {   atm = (xfs->properties[i]).name;        // Atom
	value = (xfs->properties[i]).card32;    
        printf ("%ld. %s = %ld\n", atm, XGetAtomName(display, atm), value);
    }
*/
    wid = 8;
    status = False;
    atm = XInternAtom(display, "AVERAGE_WIDTH", True);
    if (atm != None && XGetFontProperty(xfs, atm, &value))
               wid =  value/10;

    if (wid != 8) goto Ret;
        
    hei = 13;
    atm = XInternAtom(display, "PIXEL_SIZE", True);
    if (atm != None && XGetFontProperty(xfs, atm, &value))
               hei =  value;
    if (hei <10 || hei > 16) goto Ret;

    Foundry = "";
    atm = XInternAtom(display, "FOUNDRY", True);
    if (atm != None && XGetFontProperty(xfs, atm, &value))
               Foundry = XGetAtomName(display, (Atom)value);
	       
    Weight = "";
    atm = XInternAtom(display, "WEIGHT_NAME", True);
    if (atm != None && XGetFontProperty(xfs, atm, &value))
               Weight = XGetAtomName(display, (Atom)value);
	       
    Charset = "";
    atm = XInternAtom(display, "CHARSET_REGISTRY", True);
    if (atm != None && XGetFontProperty(xfs, atm, &value))
               Charset = XGetAtomName(display, (Atom)value);
	 
    
    sprintf (target, "%s %ldx%ld %s (%s)", Foundry, wid, hei,
                                 Weight, Charset);
				 
    status = True;				 

Ret:

    if (Foundry) XFree(Foundry);
    if (Charset) XFree(Charset);
    if (Weight) XFree(Weight);
    return status;    
    
}

char *egaLoadFont(char *fontName, char *fontDir, char *message)
{
    char *DefaultFonts[] =
        { NULL,
	  "-*-fixed-bold-r-*-*-16-*-*-*-*-80-*-*",
	  "-*-fixed-bold-r-*-*-13-*-*-*-*-80-*-*",
	  "-*-fixed-*-r-*-*-16-*-*-*-*-80-*-*",
	  "-*-fixed-*-r-*-*-13-*-*-*-*-80-*-*",
          "fixed", NULL 
	};
    char *fntNameCur;
    char **fntNamePtr;
    XFontStruct  *xfs;

    fntNamePtr = DefaultFonts;
    if (fontName)  DefaultFonts[0] = fontName; 
    else   fntNamePtr++;
    
    while ((fntNameCur = *fntNamePtr++) != NULL)
    {   xfs = XLoadQueryFont( display,  fntNameCur);
        if (xfs && GetFontInfoAndCheck(xfs, fontName))
	   return (char *)xfs;
    }
    
    strcpy(message, "Could not find appropriate fonts");
    return (char *) NULL;
    	   
}   


int egaShowCharSpecial(char textChar, int x, int y, int fgcol, int bgcol,
                       int hscale, int vscale, int flags)
{  

    int wid, hei, thei, twid;
    XGCValues xgcv;
    XImage   *xim;
    int      x1, y1, x2, y2, x3, y3, i, j;
    COLOUR_REF pixel, pixel1;
     
    if (!FontData) return 1;

    wid = XTextWidth(FontData, &textChar, 1);
    hei = FontHeight;
    twid = wid * hscale;
    thei = hei * vscale;

    xgcv.foreground = PixValues[fgcol];
    xgcv.background = PixValues[bgcol];
  
    XChangeGC(display, graphContext, GCForeground | GCBackground ,
                                     &xgcv);

    XDrawImageString (display, screenContext, graphContext,
                    x, y+FontData->ascent, &textChar, 1);


    xim = XGetImage(display, screenContext, x, y,
                          twid, thei, AllPlanes, ZPixmap);

    if (!xim) return 2;


    /* flip */
    y2 = hei >> 1;
    x2 = wid >> 1;
    if (flags & EGAFONT_HFLIP)
    {   
       for (y3=0; y3< hei; y3++)
         for (x3=0, x1=wid-1; x3 < x2; x3++, x1--)       
         {  pixel = XGetPixel(xim, x3, y3);
	    pixel1 = XGetPixel(xim, x1, y3);
            XPutPixel(xim, x3, y3, pixel1);
            XPutPixel(xim, x1, y3, pixel);
         }
    }	 	    
        
    if (flags & EGAFONT_VFLIP)
    {   
       for (x3=0; x3< wid; x3++)
         for (y3=0, y1=hei-1; y3 < y2; y3++, y1--)       
         {  pixel = XGetPixel(xim, x3, y3);
	    pixel1 = XGetPixel(xim, x3, y1);
            XPutPixel(xim, x3, y3, pixel1);
            XPutPixel(xim, x3, y1, pixel);
         }
    }	 	    

    /* Now scale! */
    for (y3=hei-1; y3>=0; y3--)
    {  y1 = y3*vscale;

       // From a single line
       for (x3=wid-1; x3>=0; x3--)
       {  pixel = XGetPixel(xim, x3, y3);

	  
	  // Spread to whole square          
	  x1 = x3 * hscale;
	  y2 = y1;
	  for (j = 0; j < vscale; j++)
	  {   x2 = x1;
	      for (i=0; i < hscale; i++)
	         XPutPixel(xim, x2++, y2, pixel);
	      y2++;	 
	  }      
	  
       }
    }   
             	       	       

    XPutImage(display, screenContext, graphContext, xim, 0, 0,
                       x, y, twid, thei);

    XDestroyImage(xim);
            
   if (DoInvalidate)
   { RECT rect;

     rect.left = x; 
     rect.top = y; //FontData->descent;
     rect.bottom = y + thei;
     rect.right = rect.left + twid;
     ContextRectToScreen(&rect);
   }     

   return 0;
    

}

XImage *DrawCharOnXImage(char textchar, unsigned long fgrPix, unsigned long bgrPix)
{  
   Pixmap tmpPixmap;    
   XImage *xim;    
   XGCValues xgcv;    
   int   wid, hei;

   if (!FontData) return (XImage *)NULL;

   wid = XTextWidth(FontData, &textchar, 1);
   hei = FontHeight;
   
   tmpPixmap = XCreatePixmap(display, window, wid, hei, ScreenDepth);
   if (tmpPixmap == None) return (XImage *)NULL;


   xgcv.foreground = fgrPix;
   xgcv.background = bgrPix;
  
   XChangeGC(display, graphContext, GCForeground | GCBackground ,
                                     &xgcv);


   XDrawImageString (display, tmpPixmap, graphContext,
                    0, FontData->ascent, &textchar, 1);


   xim = XGetImage(display, tmpPixmap, 0, 0, wid, hei, AllPlanes, ZPixmap);

   XFreePixmap(display, tmpPixmap);
   
   return xim;   
}
