/*===============================================================*/
/*  itetris standard header file                                 */
/*===============================================================*/
#ifdef HAVE_CONFIG_H
#include "tet_autoconf.h"
#else
#include "tet_stdconf.h"
#endif

/* Location of Pause text and Play List */
/* Default location of high scores file is now defined in Makefile */
#define PAUSE_FILE_NAME   "~/.itetris.pause"
#define BGLIST_FILE_NAME  "~/.itetris.bgl"    /* or NULL */

/* TABLE_WIDTH is the width of score table in char-s (8 pix. per char) */
#define TABLE_WIDTH   14

/* Text and background, highlighting and scroller colour numbers */
/*     See man page for vga_setegacolor.        */
#define TEXT_COLOUR    14        // Yellow
#define BKGR_COLOUR     0        // Black
#define HILITE_COLOUR  13        // Light magenta
#define SCROLL_COLOUR   2        // Green

/* Vertical distance between rows in status table */
#define ROW_DISTANCE   3

/* CONSOLE_FONT must specify an unpacked console font */
/* If font is font found, a .gz extension will be attempted */
/* Don't forget to put '/' at the end of CONSOLE_FONT_DIR */
#define  CONSOLE_FONT_DIR   "/usr/lib/kbd/consolefonts/"
#define  DEFAULT_FONT  "default8x16"
#define  DEFAULT_X_FONT  "8x13bold"

/* Default video mode. Select one of the following:
   EGA_MODE, LOW_MODE,  VGA_MODE,  SVGA_MODE.
   Use VGA_MODE if your SVGA card is not supported or badly
   supported by svgalib
*/
#define  DEFAULT_VIDEO  SVGA_MODE

/* Defines for COLOUR_NAME_FILE, COLOUR_NAME_FILE_ALT and
   DEFAUT_BG_RGB are not used in EGA mode */

/* Path to the file with colour names */
#define COLOUR_NAME_FILE  "/etc/X11/rgb.txt"
/* Alternative file - may be omitted */
#define COLOUR_NAME_FILE_ALT "/usr/lib/X11/rgb.txt"

/* Default RGB for background colour */
#define DEFAULT_BG_RGB   ""          /* Do not set  */
/* #define DEFAULT_BG_RGB "03f0000"   - take 0RRGGBB colour */
/* #define DEFAULT_BG_RGB "maroon"   - look in COLOUR_NAME_FILE */

/* Default loop mode for background music */
#define DEFAULT_LOOP_MODE   1	    /* Enable loop */	

#ifdef HAVE_LINUX_JOYSTICK_H

/*
   Joystick support: 
   -------------------------------------------------------------------
   Parameter JOYSTICK SUPPORTED is set to:
   1 - include event-driven support, provided by driver version 1x. 
   0 - include only "traditional" support provided by original driver
   -------------------------------------------------------------------
   Comment out JOYSTICK_SUPPORTED to disable joystick support
*/   
#define JOYSTICK_SUPPORTED 1 

/* If JOYSTICK_SUPPORT = 1, you need the following lines to specify
   the the threshold values for joystick axes (max. 32767)
   E.g. NEW_JOYSTICK_THRESHOLD_X = 10000 means that
     X-axis position <= -10000 is interpreted as joystick left
     X-axis position >= +10000 is interpreted as joystick right
     Position between -10000 and +10000 - no shift along X axis
*/  
#define NEW_JOYSTICK_THRESHOLD_X 23000
#define NEW_JOYSTICK_THRESHOLD_Y 23000


/* Default enabled status for joystick */
#define JOY_ENABLED_DEFAULT  0


#endif

/* Uncomment this to make life easier to your computer */
/* #define SCROLLER_DISABLED */

/* Timer ticks needed to load a backround music file (1 sec = 18.2 ticks) */
#define LOAD_DELAY 7
/* Timer ticks needed to wait after pevious player is killed */
#define KILL_DELAY 7 

/* Starting Bottom mode B_SCROLLER, B_HELP or B_NONE */
#define START_BOTTOM_DEMO       B_SCROLLER
#define START_BOTTOM_GAME       B_HELP

/* Textmode clear screen sequence (Standard ANSI should always be OK,
   so that you probably should leave it unchanged). */
#define CLEAR_SCREEN "\033[2J"

/* The 256c owl image looks ugly in 16c mode, so you better disable it
   commenting out the following line ... and reduce the size of binary  */
#define SHOW_256C_IMAGE

/* Uncomment next line for a language with right-to-left writing */
/* You may wish to uncomment also the next line,
   to display top scorers list in native language */
/* #define RIGHT_ADJUSTMENT  */
/* #define TOP_SCORES_RIGHT_ADJUSTMENT */


/* Replace with your language */
#define SCORE_TEXT    "Score:"
#define LEVEL_TEXT    "Level:"
#define LAYERS_TEXT   "Layers:"
#define BONUS_TEXT    "Bonus:"
#define NEXT_TEXT     "Next"
#define JOYSTICK_TEXT "Joystick"
#define TOTAL_TEXT    "Total:"
#define TOP_HEADER    "Top Scorers"


